@@ -8,6 +8,27 @@
 // (C) 2004 Novell (http://www.novell.com)
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.Globalization;
 using System.Text;
@@ -65,6 +86,11 @@ namespace Mono.Security.X509.Extensions {
 
 		public KeyUsageExtension (X509Extension extension) : base (extension) {}
 
+		public KeyUsageExtension () : base ()
+		{
+			extnOid = "2.5.29.15";
+		}
+
 		protected override void Decode () 
 		{
 			ASN1 bitString = new ASN1 (extnValue.Value);
@@ -75,6 +101,18 @@ namespace Mono.Security.X509.Extensions {
 				kubits = (kubits << 8) + bitString.Value [i++];
 		}
 
+		protected override void Encode ()
+		{
+			if (extnValue == null) {
+				extnValue = new ASN1 (0x03, new byte[] { 0x00, (byte)kubits });
+			}
+		}
+
+		public KeyUsages KeyUsage {
+			get { return (KeyUsages) kubits; }
+			set { kubits = Convert.ToInt32 (value, CultureInfo.InvariantCulture); }
+		}
+
 		public override string Name {
 			get { return "Key Usage"; }
 		}
