@@ -11,12 +11,12 @@
 // modify it under the terms of the GNU Lesser General Public
 // License as published by the Free Software Foundation; either
 // version 2.1 of the License, or (at your option) any later version.
-// 
+//
 // This library is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 // Lesser General Public License for more details.
-// 
+//
 // You should have received a copy of the GNU Lesser General Public
 // License along with this library; if not, write to the Free Software
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -48,7 +48,13 @@ using System.Runtime.CompilerServices;
 // You can specify all values by your own or you can build default build and revision
 // numbers with the '*' character (the default):
 
-[assembly: AssemblyVersion("0.6")]
+#if (NET_2_0)
+	[assembly: AssemblyVersion ("2.0.3600.0")]
+#elif (NET_1_1)
+	[assembly: AssemblyVersion ("1.0.5000.0")]
+#else
+	[assembly: AssemblyVersion ("1.0.3300.0")]
+#endif
 
 // The following attributes specify the key for the sign of your assembly. See the
 // .NET Framework documentation for more information about signing.
@@ -60,5 +66,5 @@ using System.Runtime.CompilerServices;
 // and in general always linking against the mono corlib seems like the right thing
 // to do.
 //
-//[assembly: AssemblyDelaySign(false)]
-//[assembly: AssemblyKeyFile("Npgsql.snk")]
+[assembly: AssemblyDelaySign (true)]
+[assembly: AssemblyKeyFile ("Npgsql/Npgsql.pub")]
