@@ -30,7 +30,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
+#if NET_2_0 || TARGET_JVM
 
 using System.ComponentModel;
 using System.Data;
@@ -54,7 +54,7 @@ namespace System.Data.Common {
 		}
 
 		protected abstract DbConnection DbConnection { get; set; }
-		protected abstract DbParameterCollection DbParameterCollection { get; set; }
+		protected abstract DbParameterCollection DbParameterCollection { get; }
 		protected abstract DbTransaction DbTransaction { get; set; }
 		public abstract bool DesignTimeVisible { get; set; }
 
@@ -72,10 +72,12 @@ namespace System.Data.Common {
 			set { Transaction = (DbTransaction) value; }
 		}
 
+#if NET_2_0
 		[MonoTODO]
 		public virtual DbCommandOptionalFeatures OptionalFeatures { 
 			get { throw new NotImplementedException (); }
 		}
+#endif
 
 		public DbParameterCollection Parameters {
 			get { return DbParameterCollection; }
@@ -109,21 +111,20 @@ namespace System.Data.Common {
 		}
 
 		public abstract int ExecuteNonQuery ();
-		public DbDataReader ExecutePageReader (CommandBehavior behavior, int startRecord, int maxRecords)
+		
+                public DbDataReader ExecutePageReader (CommandBehavior behavior, int startRecord, int maxRecords)
 		{
 			return ExecuteDbPageReader (behavior, startRecord, maxRecords);
 		}
 
-		[MonoTODO]
 		public DbDataReader ExecuteReader ()
 		{
-			throw new NotImplementedException ();
+			return ExecuteDbDataReader (CommandBehavior.Default);
 		}
 
-		[MonoTODO]
 		public DbDataReader ExecuteReader (CommandBehavior behavior)
 		{
-			throw new NotImplementedException ();
+                        return ExecuteDbDataReader (behavior);
 		}
 
 		public abstract object ExecuteScalar ();
