@@ -3,12 +3,14 @@
 //
 // Author:
 //   Tim Coleman (tim@timcoleman.com)
+//	 Boris Kirzner (borisk@mainsoft.com)
 //
 // Copyright (C) Tim Coleman, 2003
 //
 
 //
 // Copyright (C) 2004 Novell, Inc (http://www.novell.com)
+// (C) 2005 Mainsoft Corporation (http://www.mainsoft.com)
 //
 // Permission is hereby granted, free of charge, to any person obtaining
 // a copy of this software and associated documentation files (the
@@ -30,7 +32,7 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0
+#if NET_2_0 || TARGET_JVM
 
 using System.Collections;
 using System.Data.Common;
@@ -60,9 +62,9 @@ namespace System.Data.ProviderBase {
 			get { return behavior; }
 		}
 
-		[MonoTODO]
 		public override int Depth {
-			get { throw new NotImplementedException (); }
+			// default value to be overriden by user
+			get { return 0; }
 		}
 
 		[MonoTODO]
@@ -80,7 +82,9 @@ namespace System.Data.ProviderBase {
 			get { throw new NotImplementedException (); }
 		}
 
+#if NET_2_0
 		protected abstract bool IsValidRow { get; }
+#endif
 
 		[MonoTODO]
 		public override object this [[Optional] int index] {
@@ -131,10 +135,9 @@ namespace System.Data.ProviderBase {
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
 		public override void Dispose ()
 		{
-			throw new NotImplementedException ();
+			Close ();
 		}
 
 		[MonoTODO]
@@ -197,10 +200,10 @@ namespace System.Data.ProviderBase {
 			throw new NotImplementedException ();
 		}
 
-		[MonoTODO]
 		public override IEnumerator GetEnumerator ()
 		{
-			throw new NotImplementedException ();
+			bool closeReader = (CommandBehavior & CommandBehavior.CloseConnection) != 0;
+			return new DbEnumerator (this , closeReader);
 		}
 
 		[MonoTODO]
