@@ -44,6 +44,8 @@ using System.Text;
 using System.CodeDom.Compiler;
 using System.ComponentModel;
 using System.Runtime.Serialization;
+using System.Xml.Schema;
+using System.Xml.Serialization;
 
 // only for Driver
 using Microsoft.CSharp;
@@ -247,7 +249,7 @@ namespace System.Data
 			this.ds = ds;
 			this.cns = cns;
 			this.gen = gen;
-			this.opts = opts;
+			this.opts = options;
 
 			if (opts == null)
 				opts = DefaultOptions;
@@ -489,6 +491,7 @@ namespace System.Data
 			// Type
 			dsType = new CodeTypeDeclaration (opts.DataSetName (ds.DataSetName, gen));
 			dsType.BaseTypes.Add (TypeRef (typeof (DataSet)));
+			dsType.BaseTypes.Add (TypeRef (typeof (IXmlSerializable)));
 
 			// .ctor()
 			dsType.Members.Add (CreateDataSetDefaultCtor ());
@@ -509,10 +512,12 @@ namespace System.Data
 
 			// ReadXmlSerializable()
 			dsType.Members.Add (CreateDataSetReadXmlSerializable ());
+*/
 
 			// GetSchemaSerializable()
 			dsType.Members.Add (CreateDataSetGetSchemaSerializable ());
-*/
+
+			dsType.Members.Add (CreateDataSetGetSchema ());
 			dsType.Members.Add (CreateDataSetInitializeClass ());
 			dsType.Members.Add (CreateDataSetInitializeFields ());
 			dsType.Members.Add (CreateDataSetSchemaChanged ());
@@ -669,6 +674,40 @@ namespace System.Data
 			return method;
 		}
 
+		private CodeMemberMethod CreateDataSetGetSchema ()
+		{
+			CodeMemberMethod m = new CodeMemberMethod ();
+			m.PrivateImplementationType = TypeRef (typeof (IXmlSerializable));
+			m.Name = "GetSchema";
+			m.ReturnType = TypeRef (typeof (XmlSchema));
+			m.Statements.Add (Return (MethodInvoke ("GetSchemaSerializable")));
+
+			return m;
+		}
+
+		private CodeMemberMethod CreateDataSetGetSchemaSerializable ()
+		{
+			CodeMemberMethod m = new CodeMemberMethod ();
+			m.Attributes = MemberAttributes.Family | 
+				MemberAttributes.Override;
+			m.Name = "GetSchemaSerializable";
+			m.ReturnType = TypeRef (typeof (XmlSchema));
+
+			m.Statements.Add (VarDecl (typeof (StringWriter), "sw",
+				New (typeof (StringWriter))));
+			m.Statements.Add (Eval (MethodInvoke ("WriteXmlSchema", Local ("sw"))));
+			m.Statements.Add (Return (MethodInvoke (
+				TypeRefExp (typeof (XmlSchema)),
+				"Read",
+				New (typeof (XmlTextReader),
+					New (typeof (StringReader),
+						MethodInvoke (Local ("sw"),
+							"ToString"))),
+				Const (null))));
+
+			return m;
+		}
+
 		private CodeMemberMethod CreateDataSetInitializeClass ()
 		{
 			CodeMemberMethod m = new CodeMemberMethod ();
@@ -693,6 +732,7 @@ namespace System.Data
 
 			bool fkcExists = false;
 			bool ucExists = false;
+			// First the UniqueConstraints
 			foreach (DataTable dt in ds.Tables) {
 				string tname = "__table" + opts.TableMemberName (dt.TableName, gen);
 				foreach (Constraint c in dt.Constraints) {
@@ -703,19 +743,25 @@ namespace System.Data
 							ucExists = true;
 						}
 						CreateUniqueKeyStatements (m, uc, tname);
-						continue;
 					}
+				}
+			}
+			// Then the ForeignKeyConstraints
+			foreach (DataTable dt in ds.Tables) {
+				string tname = "__table" + opts.TableMemberName (dt.TableName, gen);
+				foreach (Constraint c in dt.Constraints) {
 					ForeignKeyConstraint fkc = c as ForeignKeyConstraint;
 					if (fkc != null) {
 						if (!fkcExists) {
 							m.Statements.Add (VarDecl (typeof (ForeignKeyConstraint), "fkc", null));
 							fkcExists = true;
 						}
-						CreateForeignKeyStatements (m, fkc, tname);
+						string rtname = "__table" + opts.TableMemberName (fkc.RelatedTable.TableName, gen);
+						CreateForeignKeyStatements (m, fkc, tname, rtname);
 					}
-					// What if other cases? dunno. Just ignore ;-)
 				}
 			}
+			// What if other cases? dunno. Just ignore ;-)
 			foreach (DataRelation rel in ds.Relations) {
 				string relName = opts.RelationName (rel.RelationName, gen);
 				ArrayList pcols = new ArrayList ();
@@ -735,6 +781,7 @@ namespace System.Data
 					Const (false)
 					)));
 				m.Statements.Add (Let (PropRef (FieldRef (fieldName), "Nested"), Const (rel.Nested)));
+				m.Statements.Add (MethodInvoke (PropRef ("Relations"), "Add", FieldRef (fieldName)));
 			}
 
 			return m;
@@ -756,15 +803,15 @@ namespace System.Data
 			m.Statements.Add (MethodInvoke (PropRef (FieldRef (tableField), "Constraints"), "Add", Local ("uc")));
 		}
 
-		private void CreateForeignKeyStatements (CodeMemberMethod m,ForeignKeyConstraint fkc, string tableField)
+		private void CreateForeignKeyStatements (CodeMemberMethod m,ForeignKeyConstraint fkc, string tableField, string rtableField)
 		{
 			ArrayList pcols = new ArrayList ();
-			foreach (DataColumn col in fkc.Columns)
-				pcols.Add (IndexerRef (PropRef (FieldRef (tableField), "Columns"), Const (col.ColumnName)));
+			foreach (DataColumn col in fkc.RelatedColumns)
+				pcols.Add (IndexerRef (PropRef (FieldRef (rtableField), "Columns"), Const (col.ColumnName)));
 
 			ArrayList ccols = new ArrayList ();
-			foreach (DataColumn col in fkc.RelatedColumns)
-				pcols.Add (IndexerRef (PropRef (FieldRef (tableField), "Columns"), Const (col.ColumnName)));
+			foreach (DataColumn col in fkc.Columns)
+				ccols.Add (IndexerRef (PropRef (FieldRef (tableField), "Columns"), Const (col.ColumnName)));
 
 			m.Statements.Add (Let (Local ("fkc"), New (
 				typeof (ForeignKeyConstraint),
@@ -928,6 +975,7 @@ namespace System.Data
 			c.Attributes = MemberAttributes.Assembly;
 			c.BaseConstructorArgs.Add (Const (dt.TableName));
 			c.Statements.Add (Eval (MethodInvoke ("InitializeClass")));
+			c.Statements.Add (Eval (MethodInvoke ("InitializeFields")));
 			return c;
 		}
 
@@ -957,7 +1005,10 @@ namespace System.Data
 				m.Statements.Add (Eval (MethodInvoke (
 					PropRef ("Columns"),
 					"Add",
-					New (typeof (DataColumn), Const (col.ColumnName)))));
+					New (typeof (DataColumn),
+						Const (col.ColumnName),
+						new CodeTypeOfExpression (col.DataType)
+						))));
 			}
 			return m;
 		}
