@@ -146,8 +146,6 @@ namespace System.Data
 
 	internal class XmlDataInferenceLoader
 	{
-		const string XmlnsNS = "http://www.w3.org/2000/xmlns/";
-
 		public static void Infer (DataSet dataset, XmlDocument document, XmlReadMode mode, string [] ignoredNamespaces)
 		{
 			new XmlDataInferenceLoader (dataset, document, mode, ignoredNamespaces).ReadXml ();
@@ -348,9 +346,10 @@ namespace System.Data
 			bool hasText = false;
 
 			foreach (XmlAttribute attr in el.Attributes) {
-				if (attr.NamespaceURI == XmlnsNS)
+				if (attr.NamespaceURI == XmlConstants.XmlnsNS)
 					continue;
-				if (ignoredNamespaces.Contains (attr.NamespaceURI))
+				if (ignoredNamespaces != null &&
+					ignoredNamespaces.Contains (attr.NamespaceURI))
 					continue;
 
 				hasAttributes = true;
@@ -374,7 +373,7 @@ namespace System.Data
 					XmlElement cel = n as XmlElement;
 					string childLocalName = XmlConvert.DecodeName (cel.LocalName);
 
-					switch (GetElementMappingType (cel)) {
+					switch (GetElementMappingType (cel, ignoredNamespaces)) {
 					case ElementMappingType.Simple:
 						InferColumnElement (table, cel);
 						break;
@@ -457,12 +456,13 @@ namespace System.Data
 			return col;
 		}
 
-		private ElementMappingType GetElementMappingType (XmlElement el)
+		private static ElementMappingType GetElementMappingType (
+			XmlElement el, ArrayList ignoredNamespaces)
 		{
 			foreach (XmlAttribute attr in el.Attributes) {
-				if (attr.NamespaceURI == XmlnsNS)
+				if (attr.NamespaceURI == XmlConstants.XmlnsNS)
 					continue;
-				if (ignoredNamespaces.Contains (attr.NamespaceURI))
+				if (ignoredNamespaces != null && ignoredNamespaces.Contains (attr.NamespaceURI))
 					continue;
 				return ElementMappingType.Complex;
 			}
@@ -472,18 +472,31 @@ namespace System.Data
 
 			for (XmlNode n = el.NextSibling; n != null; n = n.NextSibling)
 				if (n.NodeType == XmlNodeType.Element && n.LocalName == el.LocalName)
-					return GetElementMappingType (n as XmlElement) == ElementMappingType.Complex ? ElementMappingType.Complex : ElementMappingType.Repeated;
+					return GetElementMappingType (
+						n as XmlElement,
+						ignoredNamespaces)
+						== ElementMappingType.Complex ?
+						ElementMappingType.Complex :
+						ElementMappingType.Repeated;
 
 			return ElementMappingType.Simple;
 		}
 
 		private bool IsDocumentElementTable ()
 		{
-			XmlElement top = document.DocumentElement;
+			return IsDocumentElementTable (
+				document.DocumentElement,
+				ignoredNamespaces);
+		}
+
+		internal static bool IsDocumentElementTable (XmlElement top,
+			ArrayList ignoredNamespaces)
+		{
 			foreach (XmlAttribute attr in top.Attributes) {
-				if (attr.NamespaceURI == XmlnsNS)
+				if (attr.NamespaceURI == XmlConstants.XmlnsNS)
 					continue;
-				if (ignoredNamespaces.Contains (attr.NamespaceURI))
+				if (ignoredNamespaces != null &&
+					ignoredNamespaces.Contains (attr.NamespaceURI))
 					continue;
 				// document element has attributes other than xmlns
 				return true;
@@ -492,7 +505,8 @@ namespace System.Data
 				XmlElement el = n as XmlElement;
 				if (el == null)
 					continue;
-				if (this.GetElementMappingType (el) == ElementMappingType.Simple)
+				if (GetElementMappingType (el, ignoredNamespaces)
+					== ElementMappingType.Simple)
 					return true;
 			}
 			return false;
@@ -504,7 +518,7 @@ namespace System.Data
 		private bool IsPossibleColumnElement (XmlElement el)
 		{
 			foreach (XmlAttribute attr in el.Attributes) {
-				if (attr.NamespaceURI == "http://www.w3.org/2000/xmlns/")
+				if (attr.NamespaceURI == XmlConstants.XmlnsNS)
 					continue;
 				return false;
 			}
@@ -517,8 +531,8 @@ namespace System.Data
 }
 
 
-#region FOR_TEST
-public class Driver
+#if TEST_STANDALONE_INFERENCE
+internal class Driver
 {
 	private static void DumpDataTable (DataTable dt)
 	{
@@ -666,7 +680,7 @@ sw = Console.Out;
 	}
 }
 
-#endregion
+#endif
 
 //
 // * Design Notes
