@@ -6,6 +6,27 @@
 //		Jean-Marc Andre (jean-marc.andre@polymtl.ca)
 //
 
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+//
+
 using System;
 using System.IO;
 using System.Xml;
@@ -15,6 +36,7 @@ using System.Collections;
 using System.Runtime.Remoting;
 using System.Runtime.Serialization;
 using System.Runtime.Remoting.Messaging;
+using System.Runtime.Remoting.Metadata;
 
 namespace System.Runtime.Serialization.Formatters.Soap {
 	internal sealed class SoapReader {
@@ -172,10 +194,12 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 		private Type GetComponentType()
 		{
 			Type type = null;
-			if(GetId() != 0) return typeof(string);
 			
 			string strValue = xmlReader["type", XmlSchema.InstanceNamespace];
-			if(strValue == null) return null;
+			if(strValue == null) {
+				if(GetId() != 0) return typeof(string);
+				return null;
+			}
 			string[] strName = strValue.Split(':');
 			string namespaceURI = xmlReader.LookupNamespace(strName[0]);
 			type = mapper[new Element(string.Empty, strName[1], namespaceURI)];
@@ -205,48 +229,55 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 			long paramValuesId = NextAvailableId;
 			int[] indices = new int[1];
 
-			int initialDepth = xmlReader.Depth;
-			xmlReader.Read();
-			int i = 0;
-			while(xmlReader.Depth > initialDepth) 
+			if (!xmlReader.IsEmptyElement)
 			{
-				long paramId, paramHref;
-				object objParam = null;
-				paramNames.Add (xmlReader.Name);
-				Type paramType = null;
-				
-				if (message.ParamTypes != null) {
-					if (i >= message.ParamTypes.Length)
-						throw new SerializationException ("Not enough parameter types in SoapMessages");
-					paramType = message.ParamTypes [i];
-				}
-				
-				indices[0] = i;
-				objParam = DeserializeComponent(
-					paramType,
-					out paramId,
-					out paramHref,
-					paramValuesId,
-					null,
-					indices);
-				indices[0] = paramValues.Add(objParam);
-				if(paramHref != 0) 
+				int initialDepth = xmlReader.Depth;
+				xmlReader.Read();
+				int i = 0;
+				while(xmlReader.Depth > initialDepth) 
 				{
-					RecordFixup(paramValuesId, paramHref, paramValues.ToArray(), null, null, null, indices);
+					long paramId, paramHref;
+					object objParam = null;
+					paramNames.Add (xmlReader.Name);
+					Type paramType = null;
+					
+					if (message.ParamTypes != null) {
+						if (i >= message.ParamTypes.Length)
+							throw new SerializationException ("Not enough parameter types in SoapMessages");
+						paramType = message.ParamTypes [i];
+					}
+					
+					indices[0] = i;
+					objParam = DeserializeComponent(
+						paramType,
+						out paramId,
+						out paramHref,
+						paramValuesId,
+						null,
+						indices);
+					indices[0] = paramValues.Add(objParam);
+					if(paramHref != 0) 
+					{
+						RecordFixup(paramValuesId, paramHref, paramValues.ToArray(), null, null, null, indices);
+					}
+					else if(paramId != 0) 
+					{
+//						RegisterObject(paramId, objParam, null, paramValuesId, null, indices);
+					}
+					else 
+					{
+					}
+					i++;
 				}
-				else if(paramId != 0) 
-				{
-//					RegisterObject(paramId, objParam, null, paramValuesId, null, indices);
-				}
-				else 
-				{
-				}
-				i++;
+				xmlReader.ReadEndElement();
 			}
-
+			else
+			{
+				xmlReader.Read();
+			}
+			
 			message.ParamNames = (string[]) paramNames.ToArray(typeof(string));
 			message.ParamValues = paramValues.ToArray();
-			xmlReader.ReadEndElement();
 			RegisterObject(paramValuesId, message.ParamValues, null, 0, null, null);
 			return true;
 		}
@@ -305,6 +336,13 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 		
 		private object DeserializeArray(long id)
 		{
+			// Special case for base64 byte arrays
+			if (GetComponentType () == typeof(byte[])) {
+				byte[] data = Convert.FromBase64String (xmlReader.ReadElementString());
+				RegisterObject(id, data, null, 0, null, null);
+				return data;
+			}
+			
 			// Get the array properties
 			string strArrayType = xmlReader["arrayType", SoapTypeMapper.SoapEncodingNamespace];
 			string[] arrayInfo = strArrayType.Split(':','[',',',']');
@@ -330,13 +368,11 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 				indices[i] = array.GetLowerBound(i);
 			}
 
-
 			// Deserialize the array items
 			int arrayDepth = xmlReader.Depth;
 			xmlReader.Read();
 			while(xmlReader.Depth > arrayDepth)
 			{
-
 				Type itemType = GetComponentType();
 				if(itemType == null) 
 					itemType = array.GetType().GetElementType();
@@ -488,16 +524,25 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 			Type currentType = obj.GetType();
 			TypeMetadata tm = GetTypeMetadata (currentType);
 
-			int objDepth = xmlReader.Depth;
 			object[] data = new object[tm.MemberInfos.Length];
 			xmlReader.Read();
-			for(int i = 0; i < tm.MemberInfos.Length; i++)
+			xmlReader.MoveToContent ();
+			while (xmlReader.NodeType != XmlNodeType.EndElement)
 			{
+				if (xmlReader.NodeType != XmlNodeType.Element) {
+					xmlReader.Skip ();
+					continue;
+				}
+				
 				object fieldObject;
 				long fieldId, fieldHref;
-				int index = (int) tm.Indices[xmlReader.LocalName];
+
+				object indexob = tm.Indices [xmlReader.LocalName];
+				if (indexob == null)
+					throw new SerializationException ("Field \"" + xmlReader.LocalName + "\" not found in class " + currentType.FullName);
+				
+				int index = (int) indexob;
 				FieldInfo fieldInfo = (tm.MemberInfos[index]) as FieldInfo;
-				if(fieldInfo == null) continue;
 
 				fieldObject = 
 					DeserializeComponent(fieldInfo.FieldType,
@@ -696,8 +741,10 @@ namespace System.Runtime.Serialization.Formatters.Soap {
 			tm.MemberInfos = FormatterServices.GetSerializableMembers (type, _context);
 			
 			tm.Indices	= new Hashtable();
-			for(int i = 0; i < tm.MemberInfos.Length; i++) 
-				tm.Indices.Add (tm.MemberInfos[i].Name, i);
+			for(int i = 0; i < tm.MemberInfos.Length; i++) {
+				SoapFieldAttribute at = (SoapFieldAttribute) InternalRemotingServices.GetCachedSoapAttribute (tm.MemberInfos[i]);
+				tm.Indices [XmlConvert.EncodeLocalName (at.XmlElementName)] = i;
+			}
 			
 			_fieldIndices[type] = tm;
 			return tm;
