@@ -40,8 +40,8 @@ namespace System.Web.Compilation
 
 		public AspComponentFoundry ()
 		{
-			foundries = new Hashtable (CaseInsensitiveHashCodeProvider.Default,
-						   CaseInsensitiveComparer.Default);
+			foundries = new Hashtable (CaseInsensitiveHashCodeProvider.DefaultInvariant,
+						   CaseInsensitiveComparer.DefaultInvariant);
 
 			Assembly sw = typeof (AspComponentFoundry).Assembly;
 			RegisterFoundry ("asp", sw, "System.Web.UI.WebControls");
@@ -150,8 +150,8 @@ namespace System.Web.Compilation
 			public CompoundFoundry (string tagPrefix)
 			{
 				this.tagPrefix = tagPrefix;
-				tagnames = new Hashtable (CaseInsensitiveHashCodeProvider.Default,
-							   CaseInsensitiveComparer.Default);
+				tagnames = new Hashtable (CaseInsensitiveHashCodeProvider.DefaultInvariant,
+							   CaseInsensitiveComparer.DefaultInvariant);
 			}
 
 			public void Add (Foundry foundry)
@@ -173,6 +173,10 @@ namespace System.Web.Compilation
 			public override Type GetType (string componentName)
 			{
 				Type type = null;
+				Foundry foundry = tagnames [componentName] as Foundry;
+				if (foundry != null)
+					return foundry.GetType (componentName);
+
 				if (assemblyFoundry != null) {
 					try {
 						type = assemblyFoundry.GetType (componentName);
@@ -180,14 +184,9 @@ namespace System.Web.Compilation
 					} catch { }
 				}
 
-				Foundry foundry = tagnames [componentName] as Foundry;
-				if (foundry == null) {
-					string msg = String.Format ("Type {0} not registered for prefix {1}",
+				string msg = String.Format ("Type {0} not registered for prefix {1}",
 								     componentName, tagPrefix);
-					throw new ApplicationException (msg);
-				}
-
-				return foundry.GetType (componentName);
+				throw new ApplicationException (msg);
 			}
 		}
 	}
