@@ -27,7 +27,6 @@
 // OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-
 using System;
 using System.Collections;
 using System.Configuration;
@@ -41,7 +40,9 @@ namespace System.Web.Configuration
 		{
 			CompilationConfiguration config = new CompilationConfiguration (parent);
 
-			config.TempDirectory = AttValue ("tempDirectory", section, true);
+			string tmp = AttValue ("tempDirectory", section, true);
+			if (tmp != null && tmp != "")
+				config.TempDirectory = tmp;
 			config.DefaultLanguage = AttValue ("defaultLanguage", section);
 			if (config.DefaultLanguage == null)
 				config.DefaultLanguage = "c#";
@@ -91,13 +92,13 @@ namespace System.Web.Configuration
 				if (child.Name != "compiler")
 					ThrowException ("Unexpected element", child);
 
-				WebCompiler compiler = new WebCompiler ();
-				compiler.Languages = AttValue ("language", child);
-				compiler.Extension = AttValue ("extension", child);
+				Compiler compiler = new Compiler ();
+				compiler.Language  = AttValue ("language", child);
+				compiler.Extension  = AttValue ("extension", child);
 				compiler.Type = AttValue ("type", child);
 				compiler.CompilerOptions = AttValue ("compilerOptions", child, true, true);
 				compiler.WarningLevel = AttUIntValue ("warningLevel", child, 0);
-				config.Compilers [compiler.Languages] = compiler;
+				config.Compilers [compiler.Language] = compiler;
 			}
 		}
 
@@ -123,7 +124,6 @@ namespace System.Web.Configuration
 						continue;
 					}
 
-					aname = ShortAsmName (aname);
 					if (!assemblies.Contains (aname))
 						assemblies.Add (aname);
 
@@ -135,7 +135,6 @@ namespace System.Web.Configuration
 						config.AssembliesInBin = false;
 						continue;
 					}
-					aname = ShortAsmName (aname);
 					assemblies.Remove (aname);
 					continue;
 				}
@@ -188,14 +187,6 @@ namespace System.Web.Configuration
 			return HandlersUtil.ExtractAttributeValue (name, node, true);
 		}
 
-		private static string ShortAsmName (string long_name)
-		{
-			int i = long_name.IndexOf (',');
-			if (i < 0)
-				return long_name + ".dll";
-			return long_name.Substring (0, i) + ".dll";
-		}
-		
 		static void ThrowException (string message, XmlNode node)
 		{
 			HandlersUtil.ThrowException (message, node);
