@@ -40,12 +40,18 @@ namespace System.Web.UI.WebControls
 	[ToolboxData("<{0}:CompareValidator runat=\"server\""
 	             + "ErrorMessage=\"CompareValidator\"></{0}:CompareValidator>")]
 	public class CompareValidator: BaseCompareValidator
+#if NET_2_0
+		, IStaticTextControl
+#endif
 	{
 		public CompareValidator()
 		{
 			// Intitalize();
 		}
 
+#if NET_2_0
+    	[ThemeableAttribute (false)]
+#endif
 		[DefaultValue (""), WebCategory ("Behavior")]
 		[TypeConverter (typeof (ValidatedControlConverter))]
 		[WebSysDescription ("The ID of a control that is compared.")]
@@ -65,6 +71,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if NET_2_0
+    	[ThemeableAttribute (false)]
+#endif
 		[DefaultValue (typeof (ValidationCompareOperator), "Equal"), WebCategory ("Behavior")]
 		[WebSysDescription ("The operator that is used for comparison.")]
 		public ValidationCompareOperator Operator
@@ -84,7 +93,12 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
-		[DefaultValue (""), Bindable (true), WebCategory ("Behavior")]
+#if NET_2_0
+    	[ThemeableAttribute (false)]
+#else
+		[Bindable (true)]
+#endif
+		[DefaultValue (""), WebCategory ("Behavior")]
 		[WebSysDescription ("The value that is compared to.")]
 		public string ValueToCompare
 		{
