@@ -32,8 +32,12 @@
 using System.Collections;
 using System.Collections.Specialized;
 using System.Text;
+using System.ComponentModel;
+using System.Reflection;
 
 namespace System.Web.UI.WebControls {
+
+	[DefaultPropertyAttribute ("ControlID")]
 	public class ControlParameter : Parameter {
 
 		public ControlParameter () : base ()
@@ -68,12 +72,25 @@ namespace System.Web.UI.WebControls {
 			return new ControlParameter (this);
 		}
 		
-		[MonoTODO]
-		protected override object Evaluate (Control control)
+		protected override object Evaluate (HttpContext ctx, Control control)
 		{
-			throw new NotImplementedException ();
+			if (control == null) return null;
+			if (control.Page == null) return null;
+			
+			Control c = control.Page.FindControl (ControlID);
+			if (c == null) throw new HttpException ("Control '" + ControlID + "' not found.");
+			
+			PropertyInfo prop = c.GetType().GetProperty (PropertyName);
+			if (prop == null) throw new HttpException ("Property '" + PropertyName + "' not found in type '" + c.GetType() + "'.");
+			
+			return prop.GetValue (c, null);
 		}
 		
+		[WebCategoryAttribute ("Control")]
+		[RefreshPropertiesAttribute (RefreshProperties.All)]
+		[TypeConverterAttribute (typeof (ControlIDConverter))]
+		[DefaultValueAttribute ("")]
+		[IDReferencePropertyAttribute (typeof(System.Web.UI.Control))]
 		public string ControlID {
 			get {
 				string s = ViewState ["ControlID"] as string;
@@ -90,6 +107,9 @@ namespace System.Web.UI.WebControls {
 			}
 		}
 		
+		[DefaultValueAttribute ("")]
+		[TypeConverterAttribute (typeof (ControlPropertyNameConverter))]
+		[WebCategoryAttribute ("Control")]
 		public string PropertyName {
 			get {
 				string s = ViewState ["PropertyName"] as string;
