@@ -38,6 +38,9 @@ using System.Web.UI;
 
 namespace System.Web.UI.WebControls
 {
+#if NET_2_0
+	[ControlValuePropertyAttribute ("Text")]
+#endif
 	[DefaultProperty("Text")]
 	[Designer("System.Web.UI.Design.WebControls.LabelDesigner, " + Consts.AssemblySystem_Design, typeof (IDesigner))]
 	[DataBindingHandler("System.Web.UI.Design.TextDataBindingHandler, " + Consts.AssemblySystem_Design)]
@@ -45,15 +48,17 @@ namespace System.Web.UI.WebControls
 	[ParseChildren(false)]
 	[ToolboxData("<{0}:Label runat=\"server\">Label</{0}:Label>")]
 	public class Label : WebControl
+#if NET_2_0
+		, ITextControl
+#endif
 	{
 		public Label (): base ()
 		{
 		}
 
-		internal Label (HtmlTextWriterTag tagKey) : base (tagKey)
-		{
-		}
-
+#if NET_2_0
+		[Localizable (true)]
+#endif
 		[DefaultValue (""), Bindable (true), WebCategory ("Appearance")]
 		[PersistenceMode (PersistenceMode.InnerDefaultProperty)]
 		[WebSysDescription ("The text that should be shown on this Label.")]
@@ -66,6 +71,46 @@ namespace System.Web.UI.WebControls
 
 			set { ViewState ["Text"] = value; }
 		}
+		
+#if NET_2_0
+		[ThemeableAttribute (false)]
+		[DefaultValueAttribute ("")]
+		[IDReferencePropertyAttribute (typeof(System.Web.UI.Control))]
+		[WebCategory ("Accessibility")]
+		[TypeConverterAttribute (typeof(AssociatedControlConverter))]
+		public string AssociatedControlID
+		{
+			get {
+				object o = ViewState ["AssociatedControlID"];
+				return (o == null) ? String.Empty : (string) o;
+			}
+
+			set { ViewState ["AssociatedControlID"] = value; }
+		}
+	
+	
+		protected override HtmlTextWriterTag TagKey {
+			get {
+				if (AssociatedControlID.Length == 0)
+					return HtmlTextWriterTag.Span;
+				else
+					return HtmlTextWriterTag.Label;
+			}
+		}
+		
+		protected override void AddAttributesToRender (HtmlTextWriter writer)
+		{
+			if (AssociatedControlID.Length > 0) {
+				Control ctrl = Page.FindControl (AssociatedControlID);
+				if (ctrl != null)
+					writer.AddAttribute (HtmlTextWriterAttribute.For, ctrl.ClientID);
+			}
+			
+			base.AddAttributesToRender (writer);
+			
+		}
+#endif
+
 
 		protected override void AddParsedSubObject (object obj)
 		{
