@@ -38,6 +38,9 @@ using System.ComponentModel;
 
 namespace System.Web.UI.WebControls
 {
+#if NET_2_0
+	[ParseChildren (true, "Text")]
+#endif
 	[TypeConverter(typeof(ExpandableObjectConverter))]
 	[ControlBuilder(typeof(ListItemControlBuilder))]
 	public sealed class ListItem : IStateManager, IParserAccessor, IAttributeAccessor
@@ -50,6 +53,10 @@ namespace System.Web.UI.WebControls
 		private bool dirty_t;
 		private bool dirty_v;
 
+#if NET_2_0
+		private bool enabled;
+#endif
+
 		public ListItem(string text, string value)
 		{
 			this.text  = text;
@@ -65,6 +72,13 @@ namespace System.Web.UI.WebControls
 		{
 		}
 
+#if NET_2_0
+		public ListItem (string text, string value, bool enabled): this (text, value)
+		{
+			this.enabled = enabled;
+		}
+#endif
+
 		public static ListItem FromString(string text)
 		{
 			return new ListItem(text);
@@ -81,6 +95,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if NET_2_0
+//	    [TypeConverterAttribute (typeof (System.Web.UI.MinimizableAttributeTypeConverter))]
+#endif
 		[DefaultValue (false)]
 		public bool Selected
 		{
@@ -105,6 +122,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if NET_2_0
+		[Localizable (true)]
+#endif
 		[DefaultValue ("")]
 		[PersistenceMode (PersistenceMode.EncodedInnerDefaultProperty)]
 		public string Text
@@ -129,6 +149,9 @@ namespace System.Web.UI.WebControls
 			}
 		}
 
+#if NET_2_0
+		[Localizable (true)]
+#endif
 		[DefaultValue ("")]
 		public string Value
 		{
@@ -148,6 +171,15 @@ namespace System.Web.UI.WebControls
 					dirty_v = true;
 			}
 		}
+		
+#if NET_2_0
+		[MonoTODO ("Disable items in ListControl and subclasses")]
+		[DefaultValue (true)]
+		public bool Enabled {
+			get { return enabled; }
+			set { enabled = value; }
+		}
+#endif		
 
 		string IAttributeAccessor.GetAttribute (string key)
 		{
