@@ -3,8 +3,10 @@
 //
 // Authors:
 //	Ben Maurer (bmaurer@users.sourceforge.net)
+//	Sanjay Gupta (gsanjay@novell.com)
 //
 // (C) 2003 Ben Maurer
+// (C) 2004 Novell, Inc. (http://www.novell.com)
 //
 
 //
@@ -32,8 +34,15 @@
 using System.Collections;
 using System.Collections.Specialized;
 using System.Text;
+using System.ComponentModel;
 
 namespace System.Web.UI.WebControls {
+
+	[ParseChildrenAttribute (true)]
+	[PersistChildrenAttribute (false)]
+	[DefaultPropertyAttribute ("SelectQuery")]
+	[DesignerAttribute ("System.Web.UI.Design.WebControls.SqlDataSourceDesigner, System.Design, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.ComponentModel.Design.IDesigner")]
+	[DefaultEventAttribute ("Selecting")]
 	public class SqlDataSource : DataSourceControl {
 
 		public SqlDataSource ()
@@ -61,6 +70,16 @@ namespace System.Web.UI.WebControls {
 				throw new ArgumentException ("viewName");
 		}
 		
+		protected virtual SqlDataSourceView CreateDataSourceView (string viewName)
+		{
+			SqlDataSourceView view = new SqlDataSourceView (this, viewName, this.Context);
+			view.DataSourceViewChanged += new EventHandler (ViewChanged);
+			if (IsTrackingViewState)
+				((IStateManager) view).TrackViewState ();			
+			return view;
+		}
+
+		
 		protected override ICollection GetViewNames ()
 		{
 			return new string [] { "DefaultView" };
@@ -68,22 +87,22 @@ namespace System.Web.UI.WebControls {
 			
 		public int Insert ()
 		{
-			return View.Insert ();
+			return View.Insert (null);
 		}
 		
 		public int Delete ()
 		{
-			return View.Delete ();
+			return View.Delete (null, null);
 		}
 		
-		public IEnumerable Select ()
+		public IEnumerable Select (DataSourceSelectArguments args)
 		{
-			return View.Select ();
+			return View.Select (args);			
 		}
 		
 		public int Update ()
 		{
-			return View.Update ();
+			return View.Update (null, null, null);
 		}
 			
 		protected override void LoadViewState (object savedState)
@@ -117,6 +136,8 @@ namespace System.Web.UI.WebControls {
 		//public virtual string SqlCacheDependency { get; set; }
 		//public virtual bool EnableCaching { get; set; }
 		
+		[DefaultValueAttribute ("")]
+//		[TypeConverterAttribute (typeof (System.Web.UI.Design.WebControls.DataProviderNameConverter)]
 		public virtual string ProviderName {
 			get {
 				string val = ViewState ["ProviderName"] as string;
@@ -126,6 +147,8 @@ namespace System.Web.UI.WebControls {
 		}
 		
 		
+		[EditorAttribute ("System.Web.UI.Design.ConnectionStringEditor, System.Design, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[DefaultValueAttribute ("")]
 		public virtual string ConnectionString {
 			get {
 				string val = ViewState ["ConnectionString"] as string;
@@ -134,6 +157,7 @@ namespace System.Web.UI.WebControls {
 			set { ViewState ["ConnectionString"] = value; }
 		}
 		
+	    [DefaultValueAttribute (SqlDataSourceMode.DataSet)]
 		public SqlDataSourceMode DataSourceMode {
 			get {
 				object val = ViewState ["DataSourceMode"];
@@ -142,45 +166,71 @@ namespace System.Web.UI.WebControls {
 			set { ViewState ["DataSourceMode"] = value; }
 		}
 				
+	    [DefaultValueAttribute ("")]
 		public string DeleteCommand {
 			get { return View.DeleteCommand; }
 			set { View.DeleteCommand = value; }
 		}
 		
+		[EditorAttribute ("System.Web.UI.Design.WebControls.ParameterCollectionEditor, System.Design, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[MergablePropertyAttribute (false)]
+		[PersistenceModeAttribute (PersistenceMode.InnerProperty)]
+		[DefaultValueAttribute (null)]
 		public ParameterCollection DeleteParameters {
 			get { return View.DeleteParameters; }
 		}
 		
+		[DefaultValueAttribute (null)]
+		[EditorAttribute ("System.Web.UI.Design.WebControls.ParameterCollectionEditor, System.Design, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[MergablePropertyAttribute (false)]
+		[PersistenceModeAttribute (PersistenceMode.InnerProperty)]
 		public ParameterCollection FilterParameters {
 			get { return View.FilterParameters; }
 		}
 		
+	    [DefaultValueAttribute ("")]
 		public string InsertCommand {
 			get { return View.InsertCommand; }
 			set { View.InsertCommand = value; }
 		}
+		
+		[PersistenceModeAttribute (PersistenceMode.InnerProperty)]
+		[DefaultValueAttribute (null)]
+		[MergablePropertyAttribute (false)]
+		[EditorAttribute ("System.Web.UI.Design.WebControls.ParameterCollectionEditor, System.Design, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
 		public ParameterCollection InsertParameters {
 			get { return View.InsertParameters; }
 		}
 
+		[DefaultValueAttribute ("")]
 		public string SelectCommand {
 			get { return View.SelectCommand; }
 			set { View.SelectCommand = value; }
 		}
 		
+		[DefaultValueAttribute (null)]
+		[PersistenceModeAttribute (PersistenceMode.InnerProperty)]
+		[MergablePropertyAttribute (false)]
+		[EditorAttribute ("System.Web.UI.Design.WebControls.ParameterCollectionEditor, System.Design, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
 		public ParameterCollection SelectParameters {
 			get { return View.SelectParameters; }
 		}
 		
+	    [DefaultValueAttribute ("")]
 		public string UpdateCommand {
 			get { return View.UpdateCommand; }
 			set { View.UpdateCommand = value; }
 		}
 		
+		[PersistenceModeAttribute (PersistenceMode.InnerProperty)]
+		[EditorAttribute ("System.Web.UI.Design.WebControls.ParameterCollectionEditor, System.Design, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a", "System.Drawing.Design.UITypeEditor, System.Drawing, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b03f5f7f11d50a3a")]
+		[MergablePropertyAttribute (false)]
+		[DefaultValueAttribute (null)]
 		public ParameterCollection UpdateParameters {
 			get { return View.UpdateParameters; }
 		}
 		
+	    [DefaultValueAttribute ("")]
 		public string FilterExpression {
 			get { return View.FilterExpression; }
 			set { View.FilterExpression = value; }
@@ -230,7 +280,7 @@ namespace System.Web.UI.WebControls {
 		SqlDataSourceView View {
 			get {
 				if (view == null) {
-					view = new SqlDataSourceView (this, "DefaultView");
+					view = new SqlDataSourceView (this, "DefaultView", this.Context);
 					view.DataSourceViewChanged += new EventHandler (ViewChanged);
 					if (IsTrackingViewState)
 						((IStateManager) view).TrackViewState ();
