@@ -66,6 +66,7 @@ namespace System.Web.UI {
 		internal int renderIndex;
 		internal bool isProperty;
 		internal ILocation location;
+		ArrayList otherTags;
 
 		public ControlBuilder ()
 		{
@@ -91,6 +92,16 @@ namespace System.Web.UI {
 			this.fileName = sourceFileName;
 		}
 
+		internal void EnsureOtherTags ()
+		{
+			if (otherTags == null)
+				otherTags = new ArrayList ();
+		}
+		
+		internal ArrayList OtherTags {
+			get { return otherTags; }
+		}
+
 		public Type ControlType {
 			get { return type; }
 		}
@@ -141,6 +152,41 @@ namespace System.Web.UI {
 			}
 		}
 
+#if NET_2_0
+		public
+#else
+		internal
+#endif
+		Type BindingContainerType {
+			get {
+				if (parentBuilder == null)
+					return typeof (Control);
+					
+				if (parentBuilder is TemplateBuilder && ((TemplateBuilder)parentBuilder).ContainerType != null)
+					return ((TemplateBuilder)parentBuilder).ContainerType;
+
+				Type ptype = parentBuilder.ControlType;
+				if (ptype == null)
+					return parentBuilder.BindingContainerType;
+
+				if (!typeof (INamingContainer).IsAssignableFrom (ptype))
+					return parentBuilder.BindingContainerType;
+
+				return ptype;
+			}
+		}
+
+		internal TemplateBuilder ParentTemplateBuilder {
+			get {
+				if (parentBuilder == null)
+					return null;
+				else if (parentBuilder is TemplateBuilder)
+					return (TemplateBuilder) parentBuilder;
+				else
+					return parentBuilder.ParentTemplateBuilder;
+			}
+		}
+
 		protected TemplateParser Parser {
 			get { return parser; }
 		}
@@ -302,7 +348,7 @@ namespace System.Web.UI {
 			if (typeof (ICollection).IsAssignableFrom (propType)) {
 				builder = new CollectionBuilder ();
 			} else if (typeof (ITemplate).IsAssignableFrom (propType)) {
-				builder = new TemplateBuilder ();
+				builder = new TemplateBuilder (prop);
 			} else {
 				builder = CreateBuilderFromType (parser, parentBuilder, propType, prop.Name,
 								 null, atts, line, fileName);
@@ -339,18 +385,17 @@ namespace System.Web.UI {
 			if (this is TemplateBuilder)
 				return;
 
-			if (!typeof (IParserAccessor).IsAssignableFrom (type)) {
+			object [] atts = type.GetCustomAttributes (typeof (ParseChildrenAttribute), true);
+			
+			if (!typeof (IParserAccessor).IsAssignableFrom (type) && atts.Length == 0) {
 				isIParserAccessor = false;
 				childrenAsProperties = true;
-			} else {
-				object [] atts = type.GetCustomAttributes (typeof (ParseChildrenAttribute), true);
-				if (atts != null && atts.Length > 0) {
-					ParseChildrenAttribute att = (ParseChildrenAttribute) atts [0];
-					childrenAsProperties = att.ChildrenAsProperties;
-					if (childrenAsProperties && att.DefaultProperty != "") {
-						defaultPropertyBuilder = CreatePropertyBuilder (att.DefaultProperty,
-												parser, null);
-					}
+			} else if (atts.Length > 0) {
+				ParseChildrenAttribute att = (ParseChildrenAttribute) atts [0];
+				childrenAsProperties = att.ChildrenAsProperties;
+				if (childrenAsProperties && att.DefaultProperty != "") {
+					defaultPropertyBuilder = CreatePropertyBuilder (att.DefaultProperty,
+											parser, null);
 				}
 			}
 		}
