@@ -52,6 +52,7 @@ namespace System.Web
 		bool _bSuppressHeaders;
 		bool _bSuppressContent;
 		bool _bChunked;
+		bool last_chunk_sent;
 		bool _bEnded;
 		bool _bBuffering;
 		bool _bHeadersSent;
@@ -65,7 +66,6 @@ namespace System.Web
 		DateTime _expiresAbsolute;
 		bool _expiresAbsoluteSet;
 
-		bool _ClientDisconnected;
 		bool closed;
 
 		string	_sContentType;
@@ -73,6 +73,7 @@ namespace System.Web
 		string	_sTransferEncoding;
 		string	_sCharset;
 		string	_sStatusDescription;
+		bool forced_charset;
 
 		HttpCookieCollection _Cookies;
 		HttpCachePolicy _CachePolicy;
@@ -93,56 +94,32 @@ namespace System.Web
 #endif
 
 		string app_path_mod = null;
-                
-		public HttpResponse (TextWriter output)
+ 
+ 		private HttpResponse ()
 		{
 			 _bBuffering = true;
-			 _bFlushing = false;
-			 _bHeadersSent = false;
-
 			 _Headers = new ArrayList ();
-
 			 _sContentType = "text/html";
-
 			 _iStatusCode = 200;
-			 _sCharset = null;
-			 _sCacheControl = null;
-
-			 _lContentLength = 0;
-			 _bSuppressContent = false;
-			 _bSuppressHeaders = false;
-			 _bClientDisconnected = false;
-
-			 _bChunked = false;
+		}
 
+		public HttpResponse (TextWriter output) : this ()
+		{
 			 _TextWriter = output;
 		}
 
-		internal HttpResponse (HttpWorkerRequest WorkerRequest, HttpContext Context)
+		internal HttpResponse (HttpWorkerRequest WorkerRequest, HttpContext Context) : this ()
 		{
 			 _Context = Context;
 			 _WorkerRequest = WorkerRequest;
-
-			 _bBuffering = true;
-			 _bFlushing = false;
-			 _bHeadersSent = false;
-
-			 _Headers = new ArrayList ();
-
-			 _sContentType = "text/html";
-
-			 _iStatusCode = 200;
-			 _sCharset = null;
-			 _sCacheControl = null;
-
-			 _lContentLength = 0;
-			 _bSuppressContent = false;
-			 _bSuppressHeaders = false;
-			 _bClientDisconnected = false;
-
-			 _bChunked = false;
 		}
 
+#if TARGET_J2EE
+	public HttpWorkerRequest WorkerRequest{
+		get{ return _WorkerRequest; }
+	}
+#endif
+
 		internal void InitializeWriter ()
 		{
 			// We cannot do this in the .ctor because HttpWriter uses configuration and
@@ -183,41 +160,39 @@ namespace System.Web
 		
 		private ArrayList GenerateHeaders ()
 		{
-			ArrayList oHeaders = new ArrayList (_Headers.ToArray ());
+			ArrayList oHeaders = new ArrayList (_Headers);
 
 			oHeaders.Add (new HttpResponseHeader ("X-Powered-By", "Mono"));
-			// save culture info, we need us info here
-			CultureInfo oSavedInfo = Thread.CurrentThread.CurrentCulture;
-			Thread.CurrentThread.CurrentCulture = CultureInfo.InvariantCulture;
 
-			string date = DateTime.Now.ToUniversalTime ().ToString ("ddd, d MMM yyyy HH:mm:ss ");
-			HttpResponseHeader date_header = new HttpResponseHeader ("Date", date + "GMT");
+#if !TARGET_J2EE
+			string date = DateTime.UtcNow.ToString ("ddd, d MMM yyyy HH:mm:ss 'GMT'", CultureInfo.InvariantCulture);
+			HttpResponseHeader date_header = new HttpResponseHeader ("Date", date);
 			oHeaders.Add (date_header);
 			
 			if (IsCached)
 				cached_response.DateHeader = date_header;
 
-			Thread.CurrentThread.CurrentCulture = oSavedInfo;
-
 			if (_lContentLength > 0) {
 				oHeaders.Add (new HttpResponseHeader (HttpWorkerRequest.HeaderContentLength,
 								      _lContentLength.ToString ()));
 			}
+#endif
 
+			// Apache2 only auto-adds 'charset=blah' for text/plain and text/html
 			if (_sContentType != null) {
-				if (_sContentType.IndexOf ("charset=") == -1) {
-					if (Charset.Length == 0) {
-						Charset = ContentEncoding.HeaderName;
-					}
+				string ctype = _sContentType;
+				if (forced_charset || _sContentType == "text/plain" || _sContentType == "text/html") {
+					if (_sContentType.IndexOf ("charset=") == -1) {
+						if (Charset.Length == 0)
+							Charset = ContentEncoding.HeaderName;
 
-					// Time to build our string
-					if (Charset.Length > 0) {
-						_sContentType += "; charset=" + Charset;
+						// Time to build our string
+						if (Charset.Length > 0)
+							ctype += "; charset=" + Charset;
 					}
 				}
 
-				oHeaders.Add (new HttpResponseHeader (HttpWorkerRequest.HeaderContentType,
-								      _sContentType));
+				oHeaders.Add (new HttpResponseHeader (HttpWorkerRequest.HeaderContentType, ctype));
 			}
 
 			if (_CachePolicy != null)
@@ -264,7 +239,11 @@ namespace System.Web
 			foreach (HttpResponseHeader oHeader in oHeaders)
 				oHeader.SendContent (_WorkerRequest);
 			
+#if !TARGET_J2EE //in J2EE env. we are setting headers into
+                //HttpServletResponse instance -> headers are
+                //still not sent
 			_bHeadersSent = true;
+#endif
 		}
 
 		public string Status
@@ -356,8 +335,15 @@ namespace System.Web
 				virtualPath = UrlUtils.Reduce (virtualPath);
 			}
 
-			if (app_path_mod != null && virtualPath.IndexOf (app_path_mod) > 0)
-				virtualPath = UrlUtils.Combine (app_path_mod, virtualPath);
+			if (app_path_mod != null && virtualPath.IndexOf (app_path_mod) < 0) {
+				string rvd = _Context.Request.RootVirtualDir;
+				string basevd = rvd.Replace (app_path_mod, "");
+
+				if (!StrUtils.StartsWith (virtualPath, basevd))
+					return virtualPath;
+
+				virtualPath = UrlUtils.Combine (rvd, virtualPath.Substring (basevd.Length));
+			}
 
 			return virtualPath;
 		}
@@ -441,6 +427,10 @@ namespace System.Web
 				if (_bHeadersSent)
 					throw new HttpException ("Headers has been sent to the client");
 
+				if (value == null)
+					value = "";
+
+				forced_charset = true;
 				_sCharset = value;
 			}
 		}
@@ -538,18 +528,15 @@ namespace System.Web
 			}
 		}
 
-		public bool IsClientConnected
-		{
+		public bool IsClientConnected {
 			get {
-				if (_ClientDisconnected)
+				if (_bClientDisconnected)
 					return false;
 
-				if (null != _WorkerRequest && (!_WorkerRequest.IsClientConnected ())) {
-					_ClientDisconnected = false;
-					return false;
-				}
+				if (null != _WorkerRequest)
+					_bClientDisconnected = (!_WorkerRequest.IsClientConnected ());
 
-				return true;
+				return !_bClientDisconnected;
 			}
 		}
       
@@ -618,9 +605,6 @@ namespace System.Web
 			}
 			
 			set {
-				if (_bHeadersSent)
-					throw new HttpException ("Headers has been sent to the client");
-
 				_bSuppressContent = true;
 			}
 		}
@@ -665,7 +649,7 @@ namespace System.Web
 			if (_bHeadersSent)
 				throw new HttpException ("Headers has been sent to the client");
 
-			switch (name.ToLower ()) {
+			switch (name.ToLower (CultureInfo.InvariantCulture)) {
 			case "content-length":
 				_lContentLength = Int64.Parse (value);
 				break;
@@ -713,16 +697,22 @@ namespace System.Web
 			Clear();
 		}
 
+		internal void SetHeadersSent (bool val)
+		{
+			_bHeadersSent = val;
+		}
+		
 		public void ClearHeaders ()
 		{
 			if (_bHeadersSent)
 				throw new HttpException ("Headers has been sent to the client");
 
 			_sContentType = "text/html";
+			forced_charset = false;
 
 			_iStatusCode = 200;
 			_sCharset = null;
-			_Headers = new ArrayList ();
+			_Headers.Clear ();
 			_sCacheControl = null;
 			_sTransferEncoding = null;
 
@@ -734,10 +724,14 @@ namespace System.Web
 
 		public void Close ()
 		{
-			if (closed && !_bClientDisconnected) {
-				_bClientDisconnected = false;
+			if (!closed && !_bClientDisconnected) {
+				if (_bChunked && !last_chunk_sent) {
+					last_chunk_sent = true;
+					_WorkerRequest.SendResponseFromMemory (s_arrChunkEnd, s_arrChunkEnd.Length);
+				}
 				_WorkerRequest.CloseConnection ();
 				_bClientDisconnected = true;
+				closed = true;
 			}
 		}
 
@@ -761,7 +755,11 @@ namespace System.Web
 				return;
 
 			if (_Context.TimeoutPossible)
+#if !TARGET_J2EE
 				Thread.CurrentThread.Abort (new StepCompleteRequest ());
+#else
+				throw new vmw.@internal.j2ee.StopExecutionException();
+#endif
 
 			Flush ();
 			_bEnded = true;
@@ -801,8 +799,7 @@ namespace System.Web
 
 						SendHeaders ();
 						length = _Writer.BufferSize;
-						if (length != 0)
-							_WorkerRequest.SendCalculatedContentLength ((int) length);
+						_WorkerRequest.SendCalculatedContentLength ((int) length);
 					} else {
 						if (_lContentLength == 0 && _iStatusCode == 200 &&
 						   _sTransferEncoding == null) {
@@ -833,8 +830,8 @@ namespace System.Web
 
 				if (length == 0) {
 					if (bFinish && _bChunked) {
-						_WorkerRequest.SendResponseFromMemory (s_arrChunkEnd,
-										s_arrChunkEnd.Length);
+						last_chunk_sent = true;
+						_WorkerRequest.SendResponseFromMemory (s_arrChunkEnd, s_arrChunkEnd.Length);
 					}
 
 					_WorkerRequest.FlushResponse (bFinish);
@@ -846,6 +843,9 @@ namespace System.Web
 				if (!_bSuppressContent && Request.HttpMethod == "HEAD")
 					_bSuppressContent = true;
 
+				if (_bSuppressContent)
+					_Writer.Clear ();
+
 				if (!_bSuppressContent) {
 					_bClientDisconnected = false;
 					if (_bChunked) {
@@ -861,9 +861,11 @@ namespace System.Web
 
 						_WorkerRequest.SendResponseFromMemory (s_arrChunkSuffix,
 										       s_arrChunkSuffix.Length);
-						if (bFinish)
+						if (bFinish) {
+							last_chunk_sent = true;
 							_WorkerRequest.SendResponseFromMemory (
 									s_arrChunkEnd, s_arrChunkEnd.Length);
+						}
 					} else {
 						_Writer.SendContent (_WorkerRequest);
 					}
@@ -966,6 +968,15 @@ namespace System.Web
 			Cookies.Add (cookie);
 		}
 
+#if NET_1_1
+		// LAMESPEC: added in a service pack for 1.1 and available in 2.0 beta
+		[MonoTODO]
+		public void TransmitFile (string filename)
+		{
+			throw new NotImplementedException ();
+		}
+#endif
+
 		private void WriteFromStream (Stream stream, long offset, long length, long bufsize)
 		{
 			if (offset < 0 || length <= 0)
@@ -994,6 +1005,11 @@ namespace System.Web
 
 		public void WriteFile (string filename, bool readIntoMemory)
 		{
+#if TARGET_J2EE
+			if ((this.Request != null) && ((filename.Length <= 2)
+					|| ((filename[0] != '\\') && (filename[1] != ':'))))
+				filename = Request.MapPath(filename);
+#endif
 			FileStream fs = null;
 			try {
 				fs = File.OpenRead (filename);
@@ -1001,7 +1017,7 @@ namespace System.Web
 				if (readIntoMemory) {
 					WriteFromStream (fs, 0, size, size);
 				} else {
-					WriteFromStream (fs, 0, size, 8192);
+					WriteFromStream (fs, 0, size, 30702);
 				}
 			} finally {
 				if (fs != null)
@@ -1012,9 +1028,14 @@ namespace System.Web
 		public void WriteFile (string filename, long offset, long size)
 		{
 			FileStream fs = null;
+#if TARGET_J2EE
+			if ((this.Request != null) && ((filename.Length <= 2)
+				|| ((filename[0] != '\\') && (filename[1] != ':'))))
+				filename = Request.MapPath(filename);
+#endif
 			try {
 				fs = File.OpenRead (filename);
-				WriteFromStream (fs, offset, size, 8192);
+				WriteFromStream (fs, offset, size, 30702);
 			} finally {
 				if (fs != null)
 					fs.Close ();
@@ -1026,7 +1047,7 @@ namespace System.Web
 			FileStream fs = null;
 			try {
 				fs = new FileStream (fileHandle, FileAccess.Read);
-				WriteFromStream (fs, offset, size, 8192);
+				WriteFromStream (fs, offset, size, 30702);
 			} finally {
 				if (fs != null)
 					fs.Close ();
@@ -1036,6 +1057,9 @@ namespace System.Web
 		[MonoTODO()]
 		internal void OnCookieAdd (HttpCookie cookie)
 		{
+#if TARGET_J2EE	//naive implementation
+			Request.Cookies.Add(cookie);
+#endif
 		}
 
 		[MonoTODO("Do we need this?")]
