@@ -9,6 +9,7 @@
 
 
 using System;
+using System.Globalization;
 using System.Threading;
 using System.Reflection;
 using System.Reflection.Emit;
@@ -21,7 +22,7 @@ namespace MonoTests.System.Reflection.Emit
 {
 
 [TestFixture]
-public class AssemblyBuilderTest : Assertion
+public class AssemblyBuilderTest
 {	
 	[AttributeUsage (AttributeTargets.Assembly)]
 	public sealed class FooAttribute : Attribute
@@ -41,6 +42,8 @@ public class AssemblyBuilderTest : Assertion
 
 	static AssemblyBuilder ab;
 
+	static ModuleBuilder mb;
+
 	string tempDir = Path.Combine (Path.GetTempPath (), "MonoTests.System.Reflection.Emit.AssemblyBuilderTest");
 
 	[SetUp]
@@ -59,7 +62,7 @@ public class AssemblyBuilderTest : Assertion
 
 		domain = Thread.GetDomain ();
 		ab = genAssembly ();
-		ab.DefineDynamicModule ("def_module");
+		mb = ab.DefineDynamicModule ("def_module");
 	}
 
 	[TearDown]
@@ -100,20 +103,19 @@ public class AssemblyBuilderTest : Assertion
 	}
 
 	public void TestEntryPoint () {
-		AssertEquals ("EntryPoint defaults to null",
-					  null, ab.EntryPoint);
+		Assert.AreEqual (null, ab.EntryPoint, "EntryPoint defaults to null");
 
 		MethodInfo mi = genEntryFunction (ab);
 		ab.SetEntryPoint (mi);
 
-		AssertEquals ("EntryPoint works", mi, ab.EntryPoint);
+		Assert.AreEqual (mi, ab.EntryPoint, "EntryPoint works");
 	}
 
 	public void TestSetEntryPoint () {
 		// Check invalid arguments
 		try {
 			ab.SetEntryPoint (null);
-			Fail ();
+			Assert.Fail ();
 		}
 		catch (ArgumentNullException) {
 		}
@@ -121,7 +123,7 @@ public class AssemblyBuilderTest : Assertion
 		// Check method from other assembly
 		try {
 			ab.SetEntryPoint (typeof (AssemblyBuilderTest).GetMethod ("TestSetEntryPoint"));
-			Fail ();
+			Assert.Fail ();
 		}
 		catch (InvalidOperationException) {
 		}
@@ -131,10 +133,10 @@ public class AssemblyBuilderTest : Assertion
 		CustomAttributeBuilder cab = new CustomAttributeBuilder (typeof (FooAttribute).GetConstructor (new Type [1] {typeof (string)}), new object [1] { "A" });
 		ab.SetCustomAttribute (cab);
 
-		AssertEquals ("IsDefined works",
-					  true, ab.IsDefined (typeof (FooAttribute), false));
-		AssertEquals ("IsDefined works",
-					  false, ab.IsDefined (typeof (AssemblyVersionAttribute), false));
+		Assert.IsTrue (ab.IsDefined (typeof (FooAttribute), false),
+			"IsDefined works");
+		Assert.IsFalse (ab.IsDefined (typeof (AssemblyVersionAttribute), false),
+			"IsDefined works");
 	}
 
 	[ExpectedException (typeof (NotSupportedException))]
@@ -188,19 +190,19 @@ public class AssemblyBuilderTest : Assertion
 	}
 
 	public void TestGetDynamicModule3 () {
-		AssertNull (ab.GetDynamicModule ("FOO2"));
+		Assert.IsNull (ab.GetDynamicModule ("FOO2"));
 
 		ModuleBuilder mb = ab.DefineDynamicModule ("FOO");
 
-		AssertEquals (mb, ab.GetDynamicModule ("FOO"));
+		Assert.AreEqual (mb, ab.GetDynamicModule ("FOO"));
 
-		AssertNull (ab.GetDynamicModule ("FOO4"));
+		Assert.IsNull (ab.GetDynamicModule ("FOO4"));
 	}
 
 #if NET_1_1
 	public void TestImageRuntimeVersion () {
 		string version = ab.ImageRuntimeVersion;
-		Assert (version.Length > 0);
+		Assert.IsTrue (version.Length > 0);
 	}
 #endif
 
@@ -316,7 +318,7 @@ public class AssemblyBuilderTest : Assertion
 			ab.DefineDynamicModule (name);
 		}
 		catch (Exception) {
-			Fail ();
+			Assert.Fail ();
 		}
 
 		name = name + "A";
@@ -476,7 +478,7 @@ public class AssemblyBuilderTest : Assertion
 		AssemblyName check = AssemblyName.GetAssemblyName (filename);
 		// no exception is thrown (file not found)
 		// because it's not AssemblyBuilder.Save job to do the signing :-/
-		AssertNull ("Token", check.GetPublicKeyToken ());
+		Assert.IsNull (check.GetPublicKeyToken (), "Token");
 	}
 
 	[Test]
@@ -495,7 +497,7 @@ public class AssemblyBuilderTest : Assertion
 		AssemblyName check = AssemblyName.GetAssemblyName (filename);
 		// no public key is inserted into the assembly
 		// because it's not AssemblyBuilder.Save job to do the signing :-/
-		AssertNull ("Token", check.GetPublicKeyToken ());
+		Assert.IsNull (check.GetPublicKeyToken (), "Token");
 	}
 
 	[Test]
@@ -510,7 +512,173 @@ public class AssemblyBuilderTest : Assertion
 
 		string filename = Path.Combine (tempDir, "StrongName_WithoutAttributes.dll");
 		AssemblyName check = AssemblyName.GetAssemblyName (filename);
-		AssertEquals ("Token", "0E-EA-7C-E6-5F-35-F2-D8", BitConverter.ToString (check.GetPublicKeyToken ()));
+		Assert.AreEqual ("0E-EA-7C-E6-5F-35-F2-D8", BitConverter.ToString (check.GetPublicKeyToken ()), "Token");
+	}
+
+	[Test]
+	[ExpectedException (typeof (NotSupportedException))]
+	public void SaveUnfinishedTypes ()
+	{
+		TypeBuilder typeBuilder = mb.DefineType ("TestType", TypeAttributes.Class | TypeAttributes.Public | TypeAttributes.Sealed | TypeAttributes.AnsiClass | TypeAttributes.AutoClass, typeof(object));
+
+		ab.Save ("def_module");
+	}
+
+	[Test]
+	public void GetModules ()
+	{
+		Module[] m;
+
+		m = ab.GetModules ();
+		Assert.IsTrue (m.Length >= 2);
+
+		// Test with no modules
+		AssemblyBuilder ab2 = genAssembly ();
+		m = ab2.GetModules ();
+	}
+
+	[Test]
+	[Category ("NotWorking")]
+	public void AssemblyName_Culture ()
+	{
+		AssemblyName assemblyName = new AssemblyName ();
+		assemblyName.Name = "AssemblyNameTest";
+		assemblyName.Version = new Version ("1.0.0.0");
+		assemblyName.CultureInfo = new CultureInfo ("en-US");
+
+		const string fullName = "AssemblyNameTest, Version=1.0.0.0, Culture=en-US, PublicKeyToken=null";
+		const string abName = "AssemblyNameTest, Version=1.0.0.0, Culture=en-US";
+
+		AssertAssemblyName (tempDir, assemblyName, abName, fullName);
+	}
+
+	[Test]
+	[Category ("NotWorking")] // bug #75246
+	public void AssemblyName_PublicKey ()
+	{
+		AssemblyName assemblyName = new AssemblyName ();
+		assemblyName.Name = "AssemblyNameTest";
+		assemblyName.Version = new Version ("1.2.3.4");
+		assemblyName.KeyPair = new StrongNameKeyPair (strongName);
+
+		Assert.IsNull (assemblyName.CultureInfo, "#1");
+		Assert.AreEqual ("AssemblyNameTest, Version=1.2.3.4", assemblyName.FullName, "#2");
+
+		const string fullName = "AssemblyNameTest, Version=1.2.3.4, Culture=neutral, PublicKeyToken=0eea7ce65f35f2d8";
+
+		AssemblyBuilder ab = AppDomain.CurrentDomain.DefineDynamicAssembly (
+			assemblyName, AssemblyBuilderAccess.Save, tempDir);
+
+		AssemblyName abName = ab.GetName ();
+		Assert.IsNotNull (abName.GetPublicKeyToken (), "#3");
+		Assert.IsTrue (abName.GetPublicKeyToken ().Length > 0, "#4");
+		Assert.IsNotNull (abName.GetPublicKey () != null, "#5");
+		Assert.IsTrue (abName.GetPublicKey ().Length > 0, "#6");
+		Assert.IsNotNull (abName.CultureInfo != null, "#7");
+#if NET_2_0
+		Assert.IsTrue (abName.CultureInfo != CultureInfo.InvariantCulture, "#8");
+		Assert.AreEqual (CultureInfo.InvariantCulture.LCID, abName.CultureInfo.LCID, "#9");
+		Assert.AreEqual (fullName, abName.FullName, "#10");
+#else
+		Assert.AreEqual (CultureInfo.InvariantCulture, abName.CultureInfo, "#11");
+		Assert.AreEqual ("AssemblyNameTest, Version=1.2.3.4, PublicKeyToken=0eea7ce65f35f2d8", abName.FullName, "#12");
+#endif
+
+		ab.Save ("AssemblyNameTest.dll");
+
+		AssemblyName bakedName = AssemblyName.GetAssemblyName (Path.Combine(
+			tempDir, "AssemblyNameTest.dll"));
+
+		Assert.IsNotNull (bakedName.GetPublicKeyToken (), "#13");
+		Assert.IsNotNull (bakedName.GetPublicKey (), "#14");
+		Assert.IsNotNull (bakedName.CultureInfo, "#15");
+
+#if NET_2_0
+		Assert.IsTrue (abName.CultureInfo != CultureInfo.InvariantCulture, "#16");
+		Assert.AreEqual (CultureInfo.InvariantCulture.LCID, abName.CultureInfo.LCID, "#17");
+#else
+		Assert.AreEqual (CultureInfo.InvariantCulture, bakedName.CultureInfo, "#18");
+#endif
+
+		Assert.AreEqual (fullName, bakedName.FullName, "#19");
+	}
+
+	[Test]
+	[Category ("NotWorking")]
+	public void AssemblyName_NoVersion ()
+	{
+		AssemblyName assemblyName = new AssemblyName ();
+		assemblyName.Name = "AssemblyNameTest";
+
+		const string fullName = "AssemblyNameTest, Version=0.0.0.0, Culture=neutral, PublicKeyToken=null";
+		const string abName = "AssemblyNameTest, Version=0.0.0.0";
+
+		AssertAssemblyName (tempDir, assemblyName, abName, fullName);
+	}
+
+	[Test]
+	[Category ("NotWorking")]
+	public void AssemblyName_Version ()
+	{
+		AssemblyName assemblyName = new AssemblyName ();
+		assemblyName.Name = "AssemblyNameTest";
+		assemblyName.Version = new Version (1, 2, 3, 4);
+
+		const string fullName = "AssemblyNameTest, Version=1.2.3.4, Culture=neutral, PublicKeyToken=null";
+		const string abName = "AssemblyNameTest, Version=1.2.3.4";
+
+		AssertAssemblyName (tempDir, assemblyName, abName, fullName);
+	}
+
+	private static void AssertAssemblyName (string tempDir, AssemblyName assemblyName, string abName, string fullName)
+	{
+		AppDomain currentDomain = AppDomain.CurrentDomain;
+		AppDomain newDomain = null;
+
+		try {
+			AssemblyBuilder ab = currentDomain.DefineDynamicAssembly (
+				assemblyName, AssemblyBuilderAccess.Save, tempDir);
+			ab.Save (assemblyName.Name + ".dll");
+
+#if NET_2_0
+			// on .NET 2.0, the full name of the AssemblyBuilder matches the 
+			// fully qualified assembly name
+			Assert.AreEqual (fullName, ab.FullName);
+#else
+			Assert.AreEqual (abName, ab.FullName);
+#endif
+
+			// load assembly in separate domain, so we can clean-up after the 
+			// test
+			newDomain = AppDomain.CreateDomain ("test2", currentDomain.Evidence,
+				currentDomain.SetupInformation);
+
+			Helper helper = new Helper (Path.Combine (tempDir, assemblyName.Name + ".dll"),
+				fullName);
+			newDomain.DoCallBack (new CrossAppDomainDelegate (helper.Test));
+		} finally {
+			if (newDomain != null) {
+				AppDomain.Unload (newDomain);
+			}
+		}
+	}
+
+	[Serializable ()]
+	private class Helper
+	{
+		private readonly string _assemblyPath;
+		private readonly string _assemblyName;
+
+		public Helper (string assemblyPath, string assemblyName)
+		{
+			_assemblyPath = assemblyPath;
+			_assemblyName = assemblyName;
+		}
+		public void Test ()
+		{
+			AssemblyName assemblyName = AssemblyName.GetAssemblyName (_assemblyPath);
+			Assert.AreEqual (_assemblyName, assemblyName.ToString ());
+		}
 	}
 }
 }
