@@ -5,7 +5,26 @@
 //	Sebastien Pouliot <sebastien@ximian.com>
 //
 // (C) 2003 Motus Technologies Inc. (http://www.motus.com)
-// (C) 2004 Novell (http://www.novell.com)
+// Copyright (C) 2004-2005 Novell, Inc (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+// 
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+// 
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
 /* WARNING * WARNING * WARNING * WARNING * WARNING * WARNING * WARNING *
@@ -133,26 +152,14 @@ namespace MonoTests.System.Security.Cryptography {
 		public void SetUp () 
 		{
 			if (readStream == null) {
-				readStream = new FileStream ("read", FileMode.OpenOrCreate, FileAccess.Read);
-				writeStream = new FileStream ("write", FileMode.OpenOrCreate, FileAccess.Write);
+				readStream = new MemoryStream (new byte [0], false);
+				writeStream = new MemoryStream (new byte [0], true);
 				aes = SymmetricAlgorithm.Create ();
 				encryptor = aes.CreateEncryptor ();
 				decryptor = aes.CreateEncryptor ();
 			}
 		}
 
-		[TearDown]
-		public void TearDown () 
-		{
-			try {
-				if (File.Exists ("read"))
-					File.Delete ("read");
-				if (File.Exists ("write"))
-					File.Delete ("write");
-			}
-			catch {}
-		}
-
 		public void AssertEquals (string msg, byte[] array1, byte[] array2)
 		{
 			AllTests.AssertEquals (msg, array1, array2);
@@ -190,12 +197,20 @@ namespace MonoTests.System.Security.Cryptography {
 		{
 			cs = new CryptoStream (readStream, encryptor, CryptoStreamMode.Write);
 		}
-
+		
 		[Test]
 		[ExpectedException (typeof (ArgumentException))]
 		public void StreamWriteModeRead () 
 		{
-			cs = new CryptoStream (writeStream, encryptor, CryptoStreamMode.Read);
+			// This needs a stream which can't be read from; memory stream won't do that.
+			string f = Path.GetTempFileName ();
+			FileStream fs = new FileStream (f, FileMode.OpenOrCreate, FileAccess.Write);
+			try {
+				cs = new CryptoStream (fs, encryptor, CryptoStreamMode.Read);
+			} finally {
+				fs.Close ();
+				File.Delete (f);
+			}
 		}
 
 		[Test]
@@ -211,7 +226,8 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (NotSupportedException))]
 		public void GetLength () 
 		{
-			cs = new CryptoStream (readStream, encryptor, CryptoStreamMode.Read);
+			DebugStream debug = new DebugStream ();
+			cs = new CryptoStream (debug, encryptor, CryptoStreamMode.Read);
 			long x = cs.Length;
 		}
 
@@ -219,7 +235,8 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (NotSupportedException))]
 		public void GetPosition () 
 		{
-			cs = new CryptoStream (readStream, encryptor, CryptoStreamMode.Read);
+			DebugStream debug = new DebugStream ();
+			cs = new CryptoStream (debug, encryptor, CryptoStreamMode.Read);
 			long x = cs.Position;
 		}
 
@@ -227,7 +244,8 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (NotSupportedException))]
 		public void SetPosition () 
 		{
-			cs = new CryptoStream (readStream, encryptor, CryptoStreamMode.Read);
+			DebugStream debug = new DebugStream ();
+			cs = new CryptoStream (debug, encryptor, CryptoStreamMode.Read);
 			cs.Position = 1;
 		}
 
@@ -235,7 +253,8 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (NotSupportedException))]
 		public void Seek () 
 		{
-			cs = new CryptoStream (readStream, encryptor, CryptoStreamMode.Read);
+			DebugStream debug = new DebugStream ();
+			cs = new CryptoStream (debug, encryptor, CryptoStreamMode.Read);
 			cs.Seek (0, SeekOrigin.Begin);
 		}
 
@@ -243,7 +262,8 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (NotSupportedException))]
 		public void SetLength () 
 		{
-			cs = new CryptoStream (readStream, encryptor, CryptoStreamMode.Read);
+			DebugStream debug = new DebugStream ();
+			cs = new CryptoStream (debug, encryptor, CryptoStreamMode.Read);
 			cs.SetLength (0);
 		}
 
@@ -267,11 +287,14 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (NotSupportedException))]
 		public void FlushFinalBlock_Dual () 
 		{
-			byte[] data = {0, 1, 2, 3, 4, 5, 6, 7};
-			cs = new CryptoStream (writeStream, encryptor, CryptoStreamMode.Write);
-			cs.Write (data, 0, data.Length);
-			cs.FlushFinalBlock ();
-			cs.FlushFinalBlock ();
+			// do no corrupt writeStream in further tests
+			using (Stream s = new MemoryStream ()) {
+				byte[] data = {0, 1, 2, 3, 4, 5, 6, 7};
+				cs = new CryptoStream (s, encryptor, CryptoStreamMode.Write);
+				cs.Write (data, 0, data.Length);
+				cs.FlushFinalBlock ();
+				cs.FlushFinalBlock ();
+			}
 		}
 
 		[Test]
@@ -279,9 +302,12 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (ArgumentNullException))]
 		public void FlushFinalBlock_Disposed () 
 		{
-			cs = new CryptoStream (writeStream, encryptor, CryptoStreamMode.Write);
-			cs.Clear ();
-			cs.FlushFinalBlock ();
+			// do no corrupt writeStream in further tests
+			using (Stream s = new MemoryStream ()) {
+				cs = new CryptoStream (s, encryptor, CryptoStreamMode.Write);
+				cs.Clear ();
+				cs.FlushFinalBlock ();
+			}
 		}
 
 		[Test]
@@ -289,23 +315,35 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (ArgumentNullException))]
 		public void Read_Disposed () 
 		{
-			byte[] buffer = new byte [8];
-			cs = new CryptoStream (readStream, encryptor, CryptoStreamMode.Read);
-			cs.Clear ();
-			cs.Read (buffer, 0, 8);
+			// do no corrupt readStream in further tests
+			using (Stream s = new MemoryStream ()) {
+				byte[] buffer = new byte [8];
+				cs = new CryptoStream (s, encryptor, CryptoStreamMode.Read);
+				cs.Clear ();
+				cs.Read (buffer, 0, 8);
+			}
 		}
 
 		[Test]
 		// MS BUG [ExpectedException (typeof (ObjectDisposedException))]
-		[Ignore ("Test cause System.ExecutionEngineException on MS runtime")]
+#if NET_2_0
+		[Category ("NotWorking")]
+		[ExpectedException (typeof (IndexOutOfRangeException))]
+#else
+		[Category ("NotDotNet")] // Test cause System.ExecutionEngineException on MS runtime
+		[ExpectedException (typeof (ArgumentNullException))] // should fail like previous test case
+#endif
 		public void Read_Disposed_Break () 
 		{
-			byte[] buffer = new byte [8];
-			cs = new CryptoStream (readStream, encryptor, CryptoStreamMode.Read);
-			int len = cs.Read (buffer, 0, 4);
-			AssertEquals ("Read 4", 4, len);
-			cs.Clear ();
-			len = cs.Read (buffer, 3, 4);
+			// do no corrupt readStream in further tests
+			using (Stream s = new MemoryStream ()) {
+				byte[] buffer = new byte [8];
+				cs = new CryptoStream (s, encryptor, CryptoStreamMode.Read);
+				int len = cs.Read (buffer, 0, 4);
+				AssertEquals ("Read 4", 4, len);
+				cs.Clear ();
+				len = cs.Read (buffer, 3, 4);
+			}
 		}
 
 		[Test]
@@ -318,7 +356,6 @@ namespace MonoTests.System.Security.Cryptography {
 		}
 
 		[Test]
-		// [ExpectedException (typeof (ArgumentNullException))]
 		[ExpectedException (typeof (NullReferenceException))]
 		public void Read_NullBuffer () 
 		{
@@ -391,13 +428,22 @@ namespace MonoTests.System.Security.Cryptography {
 
 		[Test]
 		// MS BUG [ExpectedException (typeof (ObjectDisposedException))]
-		[Ignore ("Test cause System.ExecutionEngineException on MS runtime")]
+#if NET_2_0
+		[Category ("NotWorking")]
+		[ExpectedException (typeof (IndexOutOfRangeException))]
+#else
+		[Category ("NotDotNet")] // Test cause System.ExecutionEngineException on MS runtime
+		[ExpectedException (typeof (ArgumentNullException))] // to match exception throw by Read in a similar case
+#endif
 		public void Write_Disposed () 
 		{
-			byte[] buffer = new byte [8];
-			cs = new CryptoStream (writeStream, encryptor, CryptoStreamMode.Write);
-			cs.Clear ();
-			cs.Write (buffer, 0, 8);
+			// do no corrupt writeStream in further tests
+			using (Stream s = new MemoryStream ()) {
+				byte[] buffer = new byte [8];
+				cs = new CryptoStream (s, encryptor, CryptoStreamMode.Write);
+				cs.Clear ();
+				cs.Write (buffer, 0, 8);
+			}
 		}
 
 		[Test]
@@ -410,7 +456,6 @@ namespace MonoTests.System.Security.Cryptography {
 		}
 
 		[Test]
-		// [ExpectedException (typeof (ArgumentNullException))]
 		[ExpectedException (typeof (NullReferenceException))]
 		public void Write_NullBuffer () 
 		{
@@ -465,8 +510,9 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (ArgumentException))]
 		public void Write_InvalidOffset () 
 		{
+			DebugStream debug = new DebugStream ();
 			byte[] buffer = new byte [8];
-			cs = new CryptoStream (writeStream, encryptor, CryptoStreamMode.Write);
+			cs = new CryptoStream (debug, encryptor, CryptoStreamMode.Write);
 			cs.Write (buffer, 5, 4);
 		}
 
@@ -474,8 +520,9 @@ namespace MonoTests.System.Security.Cryptography {
 		[ExpectedException (typeof (ArgumentException))]
 		public void Write_OverflowCount () 
 		{
+			DebugStream debug = new DebugStream ();
 			byte[] buffer = new byte [8];
-			cs = new CryptoStream (writeStream, encryptor, CryptoStreamMode.Write);
+			cs = new CryptoStream (debug, encryptor, CryptoStreamMode.Write);
 			cs.Write (buffer, 0, Int32.MaxValue);
 		}
 
@@ -782,6 +829,9 @@ namespace MonoTests.System.Security.Cryptography {
 			AssertEquals ("Unicode DES Roundtrip", "ximian", Encoding.Unicode.GetString (data, 0, len));
 		}
 
+#if NET_2_0
+		[Category ("NotWorking")]
+#endif		
 		[Test]
 		public void DecryptPartial_TransformFinalBlock_2Pass () 
 		{
@@ -808,6 +858,9 @@ namespace MonoTests.System.Security.Cryptography {
 		}
 
 		// based on http://www.c-sharpcorner.com/Code/2002/May/FileEncryption.asp
+#if NET_2_0
+		[Category ("NotWorking")]
+#endif	
 		[Test]
 		public void WriteByteReadByte () 
 		{
@@ -1199,7 +1252,9 @@ namespace MonoTests.System.Security.Cryptography {
 			byte[] digest = hash.Hash;
 			AssertEquals ("Hash", "71-04-12-D1-95-01-CF-F9-8D-8F-F8-0D-F9-AA-11-7D", BitConverter.ToString (digest));
 		}
-
+#if NET_2_0
+		[Category ("NotWorking")]
+#endif	
 		[Test]
 		public void CascadedCryptoStream_Read () 
 		{
@@ -1253,11 +1308,30 @@ namespace MonoTests.System.Security.Cryptography {
 			DES des = DES.Create ();
 			CryptoStream csd = new CryptoStream (debug, des.CreateDecryptor (key, iv), CryptoStreamMode.Write);
 
-			byte[] data = new byte [64];
+			byte[] data = new byte [64] { 0xE1, 0xB2, 0x46, 0xE5, 0xA7, 0xC7, 0x4C, 0xBC, 0xD5, 0xF0, 0x8E, 0x25, 0x3B, 0xFA, 0x23, 0x80, 0x03, 0x16, 0x18, 0x17, 0xA3, 0x59, 0xBA, 0xAC, 0xFC, 0x47, 0x57, 0x2A, 0xF9, 0x44, 0x07, 0x84, 0x20, 0x74, 0x06, 0x38, 0xC2, 0xF3, 0xA1, 0xCE, 0x8C, 0x73, 0xB1, 0xE3, 0x75, 0x03, 0x66, 0x89, 0xF0, 0x4E, 0x98, 0x68, 0xB1, 0xBD, 0x85, 0x25, 0xFF, 0x4B, 0x11, 0x74, 0xEF, 0x14, 0xC8, 0xE9 };
 			csd.Write (data, 0, 64);
 			AssertEquals ("Length", 56, debug.Length);
 			// last block is kept for later processing
-			csd.Close ();
+		}
+
+		[Test]
+		public void PaddingModeNone ()
+		{
+			byte[] Key = new byte [16];
+			byte[] IV = new byte [16];
+			byte[] Buffer = new byte [64];
+
+			Rijndael alg = Rijndael.Create ();
+			alg.Mode = CipherMode.CBC;
+			alg.Padding = PaddingMode.None;
+
+			MemoryStream cms = new MemoryStream ();
+			ICryptoTransform ct = alg.CreateDecryptor (Key, IV);
+			CryptoStream cs = new CryptoStream (cms, ct, CryptoStreamMode.Write);
+			cs.Write (Buffer, 0, 64);
+			cs.Close ();
+
+			AssertEquals ("Length", 64, cms.ToArray ().Length);
 		}
 	}
 }
