@@ -335,20 +335,40 @@ public class DateTimeTest : Assertion
 		long offset = TimeZone.CurrentTimeZone.GetUtcOffset(t1).Ticks / 36000000000;
 		AssertEquals("C26", offset.ToString("+#;-#;0"), t1.ToString ("%z"));
 		AssertEquals("C27", offset.ToString("+00;-00;00"), t1.ToString ("zz"));
-		AssertEquals("C28", offset.ToString("+00;-00;00") + ":00", t1.ToString ("zzz"));
+		// This does not work in, eg banglore, because their timezone has an offset of
+		// +05:30
+		//AssertEquals("C28", offset.ToString("+00;-00;00") + ":00", t1.ToString ("zzz"));
 		AssertEquals("C29", " : ", t1.ToString (" : "));
 		AssertEquals("C30", " / ", t1.ToString (" / "));
 		AssertEquals("C31", " yyy ", t1.ToString (" 'yyy' "));
 		AssertEquals("C32", " d", t1.ToString (" \\d"));
 	}
 
-	[Ignore ("need a better way to handle these tests with them running on different timezones")]
-	public void TestParseExact2 () 
+	public void TestParseExact3 ()
 	{
 		DateTime t1 = DateTime.ParseExact ("2002-02-25 04:25:13Z", "u", null);
 		t1 = TimeZone.CurrentTimeZone.ToUniversalTime(t1);
-		AssertEquals ("D07d", 04 + TimeZone.CurrentTimeZone.GetUtcOffset(t1).Hours, t1.Hour);
+		AssertEquals ("D07a", 2002, t1.Year);
+		AssertEquals ("D07b", 02, t1.Month);
+		AssertEquals ("D07c", 25, t1.Day);
+		AssertEquals ("D07d", 04, t1.Hour);
+		AssertEquals ("D07e", 25, t1.Minute);
+		AssertEquals ("D07f", 13, t1.Second);
+	}
 
+	[Test]
+	public void TestParseExact4 ()
+	{
+		// bug #60912, modified hour as 13:00
+		string s = "6/28/2004 13:00:00 AM";
+		string f = "M/d/yyyy HH':'mm':'ss tt";
+		DateTime.ParseExact (s, f, CultureInfo.InvariantCulture);
+
+		// bug #63137
+		DateTime.ParseExact ("Wed, 12 May 2004 20:51:09 +0200",
+			@"ddd, d MMM yyyy H:m:s zzz",
+			CultureInfo.CreateSpecificCulture("en-us"),
+			DateTimeStyles.AllowInnerWhite);
 	}
 
 	public void TestParseExact ()
@@ -366,12 +386,6 @@ public class DateTimeTest : Assertion
 		AssertEquals ("D05", myTicks[3], t1.Ticks);
 		t1 = DateTime.ParseExact ("02/25/2002 05:25:13", "G", null);
 		AssertEquals ("D06", myTicks[4], t1.Ticks);
-		t1 = DateTime.ParseExact ("2002-02-25 04:25:13Z", "u", null);
-		AssertEquals ("D07a", 2002, t1.Year);
-		AssertEquals ("D07b", 02, t1.Month);
-		AssertEquals ("D07c", 25, t1.Day);
-		AssertEquals ("D07e", 25, t1.Minute);
-		AssertEquals ("D07f", 13, t1.Second);
 		t1 = DateTime.ParseExact ("Monday, 25 February 2002 04:25:13", "U", null);
 		t1 = TimeZone.CurrentTimeZone.ToUniversalTime(t1);
 		AssertEquals ("D08a", 2002, t1.Year);
@@ -472,13 +486,15 @@ public class DateTimeTest : Assertion
 		AssertEquals ("E16", t2.Ticks, t1.Ticks);
 
 		// Time zones
+#if false
+		// Fails durring DST for msft and mono
 		t2 = DateTime.Today + new TimeSpan (17, 18, 0);
 		t1 = DateTime.ParseExact ("11:18AM -5", "h:mmtt z", null);
 		t1 = TimeZone.CurrentTimeZone.ToUniversalTime(t1);
 		if (!TimeZone.CurrentTimeZone.IsDaylightSavingTime(t1))
 			t1 += new TimeSpan(1, 0, 0);
 		AssertEquals ("F01", t2.Ticks, t1.Ticks);
-
+		
 		t1 = DateTime.ParseExact ("11:18AM -05:00", "h:mmtt zzz", null);
 		t1 = TimeZone.CurrentTimeZone.ToUniversalTime(t1);
 		if (!TimeZone.CurrentTimeZone.IsDaylightSavingTime(t1))
@@ -496,7 +512,8 @@ public class DateTimeTest : Assertion
 		if (!TimeZone.CurrentTimeZone.IsDaylightSavingTime(t1))
 			t1 += new TimeSpan(1, 0, 0);
 		AssertEquals ("F04", t2.Ticks, t1.Ticks);
-
+#endif
+		
 		// Options
 		t2 = DateTime.Today + new TimeSpan (16, 18, 0);
 		t1 = DateTime.ParseExact ("11:18AM -5", "h:mmtt z",
@@ -560,7 +577,20 @@ public class DateTimeTest : Assertion
 		AssertEquals("Dash03", 12, t1.Month);
 		t1=DateTime.ParseExact ("---24", "---dd" , null);
 		AssertEquals("Dash04", 24, t1.Day);
-	
+
+		// Bug 63376
+		t1 = DateTime.ParseExact ("18Aug2004 12:33:00", "ddMMMyyyy hh:mm:ss", new CultureInfo ("en-US"));
+		AssertEquals ("hh allows 12, though it's useless", 0, t1.Hour);
+
+		// Bug 74775
+		DateTime.ParseExact ("Tue, 12 Apr 2005 10:10:04 +0100",
+			"Tue, 12 Apr 2005 10:10:04 +0100", enUS);
+		try {
+			DateTime.ParseExact ("Tue, 12 Apr 2005 10:10:04 +00000",
+				"ddd, dd MMM yyyy HH':'mm':'ss zzz", enUS);
+			Fail ("z02");
+		} catch (FormatException) {
+		}
 	}
 
 	[Ignore ("need to fix tests that run on different timezones")]
@@ -668,6 +698,63 @@ public class DateTimeTest : Assertion
 //		DateTime.Parse (s, null); currently am not sure if it works for _every_ culture.
 	}
 
+
+	[Test]
+	// bug #74936
+	public void TestParse4 ()
+	{
+		try {
+			DateTime.Parse("1");
+			Fail ("#1");
+		} catch (FormatException) {
+		}
+
+		try {
+			DateTime.Parse("1000");
+			Fail ("#1");
+		} catch (FormatException) {
+		}
+
+		try {
+			DateTime.Parse("8:");
+			Fail ("#1");
+		} catch (FormatException) {
+		}
+	}
+
+	[Test]
+	public void TestParse5 ()
+	{
+		// bug #71289
+		DateTime.Parse ("Sat,,,,,, 01 Oct 1994 03:00:00",
+			CultureInfo.InvariantCulture);
+		// more example...
+		DateTime.Parse ("Sat,,, 01,,, Oct,,, ,,,1994 03:00:00",
+			CultureInfo.InvariantCulture);
+#if NET_2_0
+		try {
+			// ',' after 03 is not allowed.
+			DateTime.Parse ("Sat,,, 01,,, Oct,,, ,,,1994 03,:00:00",
+			CultureInfo.InvariantCulture);
+			Fail ("Should fail here.");
+		} catch (FormatException) {
+		}
+#endif
+
+		// bug #72788
+		DateTime dt = DateTime.Parse ("21/02/05", new CultureInfo ("fr-FR"));
+		AssertEquals (2005, dt.Year);
+		AssertEquals (02, dt.Month);
+		AssertEquals (21, dt.Day);
+
+		// don't allow 2 digit years where we require 4.
+		try {
+			DateTime.ParseExact ("05", "yyyy", CultureInfo.InvariantCulture);
+			Fail ("Reject 2 digit years for yyyy");
+		} catch (FormatException) {
+		}
+	}
+
 	[Test]
 	[ExpectedException(typeof (FormatException))]
 	public void ParseFormatException1 ()
@@ -1060,8 +1147,11 @@ public class DateTimeTest : Assertion
 	}
 
 	[Test]
+	[Category ("NotWorking")]
 	public void ParseNotExact ()
 	{
+		// The error reported is:
+		// String was not recognized as valid DateTime
 		DateTime dt = DateTime.Parse ("2004-05-26T03:29:01-07:00 foo");
 		dt = TimeZone.CurrentTimeZone.ToUniversalTime (dt);
 		AssertEquals ("DateTime.Parse not exact", 632211641410000000, dt.Ticks);
